/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.http.listeners;

import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.bc.http.HttpBCImpl;
import org.ow2.petals.bc.http.util.HttpUtil;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.component.framework.util.SourceHelper;

/**
 * HTTP BC listener for incoming JBI messages.
 *
 * @author mcarpentier - EBM Websourcing
 */
public class HttpBCJBIListener extends AbstractJBIListener {

    public static final String URL_PATHELEMENT = "url";

    private HttpRequestDescriptorBuilder requestDescriptorBuilder;

    private HttpConnectionManager connectionManager;

    public HttpBCJBIListener() {
        super();
    }

    /**
     * init
     */
    @Override
    protected final void init() {
        this.requestDescriptorBuilder = ((HttpBCImpl) this.getComponent())
                .getRequestDescriptorBuilder();

        this.connectionManager = ((HttpBCImpl) this.getComponent())
                .getConnectionManager();

    }

    /**
     * Called by the JBI channel listener when a jbi message is accepted. Build
     * the request, send it, and write the response in the message exchange
     *
     * @param exchange
     *            the {@link MessageExchangeWrapper} received
     */
    @Override
    protected final void onJBIMessage(final Exchange exchange) {
        final String faultMsg = "Can't return fault to consumer";

        // Fault ignored
        if (exchange.getFault() == null) {
            try {
                this.getLogger().info("Message received");

                final Extensions extensions = new Extensions(this.getProvides()
                        .getExtensions());
                final HttpRequestDescriptor requestDescriptor = this.requestDescriptorBuilder
                        .build(extensions, exchange);
                final String result = this.connectionManager
                        .execute(requestDescriptor);

                if (!exchange.isInOnlyPattern()) {
                    if (HttpUtil.getInstance().isXmlWellFormed(result)) {
                        exchange.setOutMessageContent(result);
                    } else {
                        exchange.setOutMessageContent(SourceHelper.createContentSource(result));
                    }

                }

            } catch (final PEtALSCDKException e) {
                try {
                    if (!exchange.isInOnlyPattern()) {
                        exchange.setFault(new Exception(e));
                    } else {
                        this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                    }
                } catch (final MessagingException e1) {
                    this.getLogger().log(Level.SEVERE, faultMsg, e);
                }
            } catch (final JBIException e) {
                try {
                    if (!exchange.isInOnlyPattern()) {
                        exchange.setFault(new Exception(e));
                    } else {
                        this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                    }
                } catch (final MessagingException e1) {
                    this.getLogger().log(Level.SEVERE, faultMsg, e);
                }
            }
        }

    }

}
