/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.http.listeners;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.ow2.petals.bc.http.HttpBCImpl;

/**
 * This class is used to process http requests
 * @author mcarpentier - EBM Websourcing
 */
public class HttpConnectionManager {

    /**
     * used to build {@link HttpRequestDescriptor}
     */
    private HttpRequestDescriptorBuilder requestDescriptorbuilder;

    /**
     * logger
     */
    private Logger log;

    /**
     * @param log
     *            the component {@link Logger}
     * @param requestDescriptorbuilder
     *            the {@link HttpRequestDescriptorBuilder} used to build
     *            {@link HttpRequestDescriptor}
     */
    public HttpConnectionManager(final Logger log,
            final HttpRequestDescriptorBuilder requestDescriptorbuilder) {
        super();
        this.log = log;
        this.requestDescriptorbuilder = requestDescriptorbuilder;
    }

    /**
     * @param requestDescriptor
     *            the {@link HttpRequestDescriptor} to process
     * @return the request response
     * @throws JBIException
     *             if an error occurs while processing the request
     */
    public final String execute(final HttpRequestDescriptor requestDescriptor)
        throws JBIException {
        // Create an instance of HttpClient.
        final HttpClient client = new HttpClient();

        // Create a method instance.
        final String fullUrl = this.requestDescriptorbuilder
                .buildRequestUrl(requestDescriptor);
        HttpMethodBase method;

        if (HttpBCImpl.HTTP_POST_METHOD.equalsIgnoreCase(requestDescriptor
                .getMethod())) {
            method = new PostMethod(fullUrl);
        } else {
            method = new GetMethod(fullUrl);
        }

        try {
            // Execute the method.
            this.log.info("requestiong URL..." + fullUrl);
            final int statusCode = client.executeMethod(method);

            if (statusCode != HttpStatus.SC_OK) {
                final String msg = "Method failed: " + method.getStatusLine();
                this.log.log(Level.SEVERE, msg);
                throw new JBIException(msg);
            }

            // Read the response body.
            final byte[] responseBody = method.getResponseBody();

            // Deal with the response.
            // Use caution: ensure correct character encoding and is not
            // binary data
            return new String(responseBody);

        } catch (final HttpException e) {
            final String msg = "Fatal protocol violation: " + e.getMessage();
            this.log.log(Level.SEVERE, msg, e);
            throw new JBIException(msg);
        } catch (final IOException e) {
            final String msg = "Fatal transport error: " + e.getMessage();
            this.log.log(Level.SEVERE, msg, e);
            throw new JBIException(msg);
        } finally {
            // Release the connection.
            method.releaseConnection();
        }

    }

}
